"""
Contains all click contexts
"""

from aicrowd.contexts.challenge import Challenge
from aicrowd.contexts.config import CLIConfig

from aicrowd.contexts.context import (
    ConfigContext,
    pass_config,
    ChallengeContext,
    pass_challenge,
)
