"""
Lists all error names and their exit codes
"""

EXIT_SUCCESS = 0

# exit codes 1 and 2 are used internally by click

INVALID_FILE = 3
INVALID_CREDENTIALS = 4
INVALID_PARAMETER = 5

CONFIG_NOT_SET = 6

UNKNOWN_ERROR = 7
