"""
Contains all commands
"""

from aicrowd.cmd.challenge import challenge_command
from aicrowd.cmd.dataset import dataset_command
from aicrowd.cmd.login import login_command
from aicrowd.cmd.version import version_command
from aicrowd.cmd.submission import submission_command
from aicrowd.cmd.notebook import notebook_command
